<?php
declare(strict_types=1);
namespace Datec\DatecAmino\Domain\Repository;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Core\Database\Query\QueryBuilder;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Extbase\Persistence\Generic\Mapper\DataMapper;
use TYPO3\CMS\Extbase\Persistence\Repository;

/**
 * A repository for Amino
 *
 */
class AminoRepository extends Repository {
	/**
	 * @var ConnectionPool
	 */
	protected $databaseConnection; 
	
	/**
	 * @var DataMapper
	 * */
	protected $dataMapper;
	
	/**
	 * @var ConfigurationManagerInterface
	 */
	protected $configurationManager;

	/**
	 * settings
	 */
	protected $settings;

    /**
     * @param DataMapper $dataMapper
     */
    public function injectDataMapper(DataMapper $dataMapper)
    {
        $this->dataMapper = $dataMapper;
    }

    /**
     * @param ConfigurationManagerInterface $configurationManager
     */
    public function injectConfigurationManagerInterface(ConfigurationManagerInterface $configurationManager)
    {
        $this->configurationManager = $configurationManager;
    }

    /**
     * Configures and supplies connection to custom database via ConnectionPool.
     *
	 * @return void
	 */
	public function initializeObject() {
		$this->settings = $this->configurationManager->getConfiguration('Settings','DatecAmino','Amino');
        $connectionName = 'aminoDB';

		$connectionPool = GeneralUtility::makeInstance(ConnectionPool::class);
		$connectionPool->getConnectionByName($connectionName);
		$this->databaseConnection = $connectionPool;

	}
	
	/**
	 * searches for land string
	 * 
	 * @param string $land
	 * @return boolean|array result array or FALSE if not found
	 */
	private function findAminoLand($land) {
		$result = FALSE;
		$table = 'land';
		$select = '*';
		$land = substr($land, 0, 1); // might be 'Y2' for bavaria, should be 'Y'
        /* @var QueryBuilder $queryBuilder */
        $queryBuilder = $this->databaseConnection->getQueryBuilderForTable($table);
		
		$res = $queryBuilder
            ->select($select)
            ->from($table)
            ->where($queryBuilder->expr()->eq('land', $queryBuilder->createNamedParameter($land)))
            ->setMaxResults(1)
            ->execute();
		
		if ($res) {
			$result = $res->fetch();
		}
		
		return $result;
	} 

	/**
	 * finds prescriptions by amino search criteria
	 * "Rezepturen"
	 * 
	 * @param \Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria
     * @param boolean $unmapped whether to map the result
	 *
	 * @return array $result
	 */
	public function findAminoPrescriptions($aminoCriteria, $unmapped = FALSE) {	
		$result = array();
		$rows = array();
		$table = $this->settings['tables']['aminoPrescription'];
		$select = '*';
        /* @var QueryBuilder $queryBuilder */
        $queryBuilder = $this->databaseConnection->getQueryBuilderForTable($table);
		
		$sword1 = $queryBuilder->escapeLikeWildcards($aminoCriteria->getSword1());
		$or = $queryBuilder->expr()->orX();
		$or->add($queryBuilder->expr()->like(
		    'stichwort',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'name',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'synonym',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'zusammensetzung',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'indikation',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'darr',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		
		$queryBuilder
            ->select($select)
            ->from($table)
            ->where($queryBuilder->expr()->gte('pubgrad', 2))
            ->andWhere($or);
		
		$sword2 = $sword1 = $queryBuilder->escapeLikeWildcards($aminoCriteria->getSword2());
		if (!empty($sword2)) {
			$orSword2 = $queryBuilder->expr()->orX();
			$orSword2->add($queryBuilder->expr()->like(
			    'stichwort',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'name',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'synonym',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'zusammensetzung',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'indikation',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'darr',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			switch($aminoCriteria->getOperant()) {
				case 'AND':
                    $queryBuilder->andWhere($orSword2);
					break;
				case 'OR':
                    $queryBuilder->orWhere($orSword2);
					break;
				case 'AND NOT':
                    $queryBuilder->andWhere('NOT ('.$orSword2->__toString().')');
					break;
			}
		}
		
		$res = $queryBuilder->execute();
		if ($res) {
			while($row = $res->fetch()) {
				$rows[] = $row; 
			}
			$result[0] = $unmapped === TRUE ? $rows : $this->dataMapper->map('\Datec\\DatecAmino\\Domain\\Model\\AminoPrescription', $rows);
			$result[1] = count($result[0]);
		} else {
			$result[0] = $res;
			$result[1] = array($select, $table, $or.(isset($orSword2) ? ' '.$orSword2 : ''));
		}

		return $result;
	}
	
	/**
	 * finds links by amino search criteria
	 * "Linkliste"
	 *
	 * @param \Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria
	 * @param boolean $unmapped whether to map the result
	 *
	 * @return array $result
	 */
	public function findAminoLinks($aminoCriteria, $unmapped = FALSE) {
		$result = array();
		$rows = array();
		$table = $this->settings['tables']['aminoLink'];
		$select = '*';
        /* @var QueryBuilder $queryBuilder */
        $queryBuilder = $this->databaseConnection->getQueryBuilderForTable($table);
	
		// first search word is required
		$sword1 = $queryBuilder->escapeLikeWildcards($aminoCriteria->getSword1());
		$or = $queryBuilder->expr()->orX();
		$or->add($queryBuilder->expr()->like(
		    'stichwort',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'title',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'thema1',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'thema2',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'thema3',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'anbieter',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'link',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );

		$queryBuilder->select($select)
            ->from($table)
            ->where($queryBuilder->expr()->gte('pubgrad', 2))
            ->andWhere($or);
	
		$sword2 = $sword1 = $queryBuilder->escapeLikeWildcards($aminoCriteria->getSword2());
		if (!empty($sword2)) {
			$orSword2 = $queryBuilder->expr()->orX();
			$orSword2->add($queryBuilder->expr()->like(
			    'stichwort',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'title',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'thema1',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'thema2',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'anbieter',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'link',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			
			switch($aminoCriteria->getOperant()) {
				case 'AND':
                    $queryBuilder->andWhere($orSword2);
					break;
				case 'OR':
                    $queryBuilder->orWhere($orSword2);
					break;
				case 'AND NOT':
                    $queryBuilder->where('AND NOT ('.$orSword2->__toString().')');
					break;
			}
			
		}
		
		$langs = $aminoCriteria->getLangs();
		if (in_array($table, explode(',',$this->settings['langOptions']['tables'])) && !empty($langs)) {
			$orLang = $queryBuilder->expr()->orX();
			foreach($langs as $key => $lang) {
				if ($lang == 'other') {
					$otherLangs = explode(',',$this->settings['langOptions']['otherLangs']);
					foreach ($otherLangs as $otherKey => $otherLang) {						
						$orLang->add($queryBuilder->expr()->eq('sprache', $otherLang));
					}
				} else {
					$orLang->add($queryBuilder->expr()->eq('sprache', $lang));
				}
			}
            $queryBuilder->andWhere($orLang);
		}
		
		$res = $queryBuilder
            ->orderBy('lfd','ASC')
            ->execute();
		if ($res) {
			while($row = $res->fetch()) {
				$rows[] = $row;
			}
			
			$result[0] = $unmapped === TRUE ? $rows : $this->dataMapper->map('\Datec\\DatecAmino\\Domain\\Model\\AminoLink', $rows);
			$result[1] = count($result[0]);
		} else {
			$result[0] = $res;
            $result[1] = array($select, $table, $or.(isset($orSword2) ? ' '.$orSword2 : '').(isset($orLang) ? ' '.$orLang : ''));
		}
	
		return $result;
	}
	
	/**
	 * finds requests by amino search criteria
	 * "Anfragen"
	 *
	 * @param \Datec\DatecAmino\Domain\Model\AminoCriteria $aminoCriteria
	 * @param boolean $unmapped whether to map the result
	 *
	 * @return array $result
	 */
	public function findAminoRequests($aminoCriteria, $unmapped = FALSE) {
		$result = array();
		$rows = array();
		$table = $this->settings['tables']['aminoRequest'];
		$select = '*';
		$where = 'pubgrad >= 2';
        /* @var QueryBuilder $queryBuilder */
        $queryBuilder = $this->databaseConnection->getQueryBuilderForTable($table);

		// first search word is required
		$sword1 = $sword1 = $queryBuilder->escapeLikeWildcards($aminoCriteria->getSword1());
		
		$or = $queryBuilder->expr()->orX();
		$or ->add($queryBuilder->expr()->like(
		    'stichwort',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'thema',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'kommentar',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
		$or->add($queryBuilder->expr()->like(
		    'frage',
            $queryBuilder->createNamedParameter('%'.$sword1.'%'))
        );
	
		$queryBuilder->select($select)
            ->from($table)
            ->where($queryBuilder->expr()->gte('pubgrad',2))
            ->andWhere($or);
		
		$sword2 = $sword1 = $queryBuilder->escapeLikeWildcards($aminoCriteria->getSword2());
		if (!empty($sword2)) {
			$orSword2 = $queryBuilder->expr()->orX();
			$orSword2->add($queryBuilder->expr()->like(
			    'stichwort',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'thema',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'kommentar',
                $queryBuilder->createNamedParameter('%'. $sword2.'%'))
            );
			$orSword2->add($queryBuilder->expr()->like(
			    'frage',
                $queryBuilder->createNamedParameter('%'.$sword2.'%'))
            );
			
			switch($aminoCriteria->getOperant()) {
				case 'AND':
                    $queryBuilder->andWhere($orSword2);
					break;
				case 'OR':
                    $queryBuilder->orWhere($orSword2);
					break;
				case 'AND NOT':
                    $queryBuilder->where('AND NOT ('.$orSword2->__toString().')');
					break;
			}
		}
	
		$res = $queryBuilder->execute();
		if ($res) {
			while ($row = $res->fetch()) {
				if ($this->settings['loadLandNames']) {
					$land = $this->findAminoLand($row['land']);
					if ($land) {
						$row['land'] = $land['name'];
					}
				}
				$rows[] = $row;
			}
			$result[0] = $unmapped === TRUE ? $rows : $this->dataMapper->map('\Datec\\DatecAmino\\Domain\\Model\\AminoRequest', $rows);
			$result[1] = count($result[0]);
		} else {
			$result[0] = $res;
			$result[1] = array($select, $table, $where);
		}
	
		return $result;
	}
		
}